#ifndef _AUDIO_
#define _AUDIO_ 1
/*!
 * @brief サウンド関係の処理を行うプログラムのヘッダ
 * @author Fujii Yasuhisa
 * @date 07/30/2009
 */

#include <fcntl.h>
#include <sys/soundcard.h>
#include <cstdio>
#include <sys/ioctl.h>
#include <unistd.h>

namespace slp {
namespace spojus {

class Audio {
public:
	Audio() {
		Init();
	}
	~Audio() {}

	void Init() {
		fmt_     = AFMT_S16_LE;
		freq_    = 16000;
		channel_ = 1;
	}

protected:
	void Setup();

	int fmt_;
	int freq_;
	int channel_;

	int fd_;		///< ファイル記述子
};

class Mic : public Audio {
public:
	Mic() {}
	~Mic() {}

	void Init() {
		Open();
	}

	/*!
	 * @brief サウンドデバイスからデータを読み込む
	 * @param short *buf 読み込んだデータを格納するバッファ
	 * @param size_t size データを読み込むバイト数
	 * @retval int 実際に読み込まれたバイト数
	 */
	int Read(short *buf, size_t size);

	/*!
	 * @brief サウンドデバイスを読み込み専用でオープンする
	 * @retval void
	 */
	void Open();

	/*!
	 * @brief サウンドデバイスをクローズする
	 * @retval void
	 */
	void Close();
};

}
}

#endif // #ifndef _AUDIO_
