#ifndef _FAIRING_
#define _FAIRING_

/**
 * @brief Confusion Networkをから整形文を推定するためのプログラムのヘッダ達
 *                                 @author Fujii Yasuhisa
 *                                 @date 1/28/2009
 */

#include <deque>
#include <list>
#include <map>

#include "my_typedefs.hpp"
#include "conf_net.hpp"
#include "dictionary.hpp"
#include "lang_model.hpp"

namespace slp {
namespace spojus {

class Fairing {
private:
	struct HypWord {
		unsigned int word;
		int stime;
		int etime;
	};

	struct Hyp {
		deque<HypWord> hypwords;
		Rnum score;
		Rnum heuristics;
	};

public:

	Fairing() {
		InitTranslationScore();
	}

	Fairing(ConfNet *conf_net, Spojus *spojus_) {
		conf_net_ = conf_net;
		spojus_ = spojus;

		Init();
//		DynamicDelPrior = 1;
		DynamicDelPrior = 0;
		DelPriorScalingFactor = 6.0;
		PriorWeight = 0.5;

		sNgram = slm->getNgram();
		wNgram = wlm->getNgram();

		string puncname = "、+、+記号-読点";
		TenIndex = dict->name2wordid(puncname);

		puncname = "。+。+記号-句点";
		MaruIndex = dict->name2wordid(puncname);

		lweight = 20.0;

		Nbest = 2000;

		InterpolationWeight = 0.75;

		initTranslationScore();
	}

	virtual ~Fairing(){}

	void setDelPrior(Rnum DelPrior) {
		this->DelPrior = DelPrior;
	}

	void setPriorWeight(Rnum PriorWeight) {
		this->PriorWeight = PriorWeight;
	}

	void setDelPriorScalingFactor(Rnum DelPriorScalingFactor) {
		this->DelPriorScalingFactor = DelPriorScalingFactor;
	}

	void setInterpolationWeight(Rnum InterpolationWeight) {
		this->InterpolationWeight = InterpolationWeight;
	}

	void search();
	void show_N_best(list<Hyp> &hyp, int N);

	unsigned int TenIndex;
	unsigned int MaruIndex;

private:

	void initTranslationScore();
	void computeTranslationScore(int);

	void init_conf_net();
	void recompute_del_score(int);
	void expansion_hyp(list<Hyp> &cur_hyp, list<Hyp> &pre_hyp, int arc);
	static bool compare_hyp_words(list<Hyp>::iterator c_iter, Hyp &new_hyp, unsigned short N);
	void create_new_hyp(Hyp &new_hyp, list<Hyp>::iterator ph_iter, int arc);

	int findArcInNode(int node);

private:
	ConfNet *conf_net_;	// コンフュージョンネットワーク
	Spojus *spojus_;

	map<string, Rnum> translation_score_map_;

	Word lm_word_[10];

	Rnum PriorWeight;
	Rnum DelPrior;
	Rnum DelPriorScalingFactor;
	char DynamicDelPrior;
};

}
}

#endif // #ifndef _FAIRING_
