#ifndef _HMM_
#define _HMM_

/*!
 * @brief HMMを扱うためのプログラムのヘッダ
 * @author Fujii Yasuhisa
 * @date 23/03/2009
 */

#include <string>
#include <vector>

#include "my_typedefs.hpp"
#include "gmm.hpp"

namespace slp {
namespace hcnf {
class State;
}
namespace spojus {

class Hmm {
public:
	Hmm(){};
	Hmm(std::string &model_name) { model_name_ = model_name; };
	virtual ~Hmm() {};

	void set_model_name_(const std::string& model_name) { model_name_ = model_name; }
public:

	class Transition {
	public:
		Transition(int state_num) {
			state_num_ = state_num;
			matrix_ = new Rnum *[state_num];
			for (int i = 0; i < state_num; i ++) {
				matrix_[i] = new Rnum[state_num];
				for (int j = 0; j < state_num; j ++) {
					matrix_[i][j] = -INFINITY;
				}
			}
		}

		virtual ~Transition() {
			for (int i = 0; i < state_num_; i ++) {
				delete [] matrix_[i];
			}
			delete [] matrix_;
		}

	public:
		int state_num_;
		Rnum **matrix_;
	};

	class State {
	public:
		// gmm_ == Null でtransitionを表現
		State() : stream_gmm_(NULL), type_(kNormal) {}
		virtual ~State() {}

	public:
		StreamGmm *stream_gmm_;				// tiedモデルなどに対応するためにポインタにしておく
		enum Type {kNormal, kInitial, kFinal};
		Type type_;
	};

public:
	typedef std::vector<State *> StateContainer;

public:
	StateContainer state_;
	Transition *transition_;
	std::string model_name_;
#ifdef HCNF
	hcnf::State* hcnf_state_;
#endif
};

}
}

#endif // #ifndef _HMM_
