#ifndef _NETWORK_
#define _NETWORK_ 1
/**
 * @brief Network関係のプログラム達のヘッダファイル達
 *                          @author Fujii Yasuhisa
 *                          @date 28/1/2009
 */

#include <iostream>
#include <cstdio>
#include <errno.h>
#include <sys/types.h>
#include <string.h>

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif

namespace slp {
namespace spojus {

class MySocket {
protected:
	static const int ReadBuffLength = 1000;
	static const int WriteBuffLength = 1000;

public:
	MySocket() {
		memset((char *)&server_, 0, sizeof(server_));
                server_.sin_family = AF_INET;
#ifdef WIN32
		WSADATA wsa_data;
		WSAStartup(MAKEWORD(2, 0), &wsa_data);
		sfd_ = socket(AF_INET, SOCK_STREAM, 0);
		if (sfd_ == INVALID_SOCKET) {
			perror("Can't create socket");
		}
#else
		sfd_ = socket(AF_INET, SOCK_STREAM, 0);
		if (sfd_ < 0) {
			perror("Can't create socket");
		}
#endif
	};
	virtual ~MySocket() {
#ifdef WIN32
		WSACleanup();
#else
		close(sfd_);
#endif
	};

	virtual void write(const char *, int){};
	virtual int read(char *){ return 0; };
	virtual int read(char *, int){ return 0; };
	virtual void accept() {};

	void GetLine(std::string& line);

protected:
#ifdef WIN32
	SOCKET sfd_;
#else
	int sfd_;	// ソケットディスクリプタ
#endif
	struct sockaddr_in server_;
private:
};

	
class MyClient : public MySocket {
public:
	MyClient(){};
	MyClient(const std::string& hostname, in_port_t port) {
		connect(hostname, port);
	}
	virtual ~MyClient(){};

	void write(const char *, int);
	int read(char *);
	int read(char *, int);

	int connect(const std::string& hostname, in_port_t port);

private:
	
};

class MyServer : public MySocket {
public:

	MyServer(){};
	MyServer(in_port_t port) {
		bind(port);
		listen();
	}

	virtual ~MyServer(){
		close(sfd_accept_);
	};

	void write(const char *, int);
	int read(char *);
	int read(char *, int);

	void bind(in_port_t port);
	void listen();
	void accept();
private:
#ifdef WIN32
	SOCKET sfd_accept_;
#else
	int sfd_accept_;
#endif
};

}
}

#endif // #ifndef _NETWORK_

