#ifndef _UTILITY_
#define _UTILITY_ 1
/*!
 * @brief SPOJUS用汎用関数のヘッダ
 * @author Fujii Yasuhisa
 * @date 05/28/2009
 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <stdexcept>

namespace Utility 
{
/*!
 * @brief 受け取ったストリームについてEOFが入力されたかどうかをチェックする
 * @param std::istream& in
 * @retval bool
 */
bool check_eof(std::istream& in);

/*!
 * @brief endianness判別関数
 * @retval int : 0 -> little endian | 1 -> big endian
 */
int endianness_judge();

/*!
 * @brief 圧縮ファイルに対応したopen
 *
 * 拡張子が.gzで保存されたファイルはgunzipで解答してpopenする
 * 普通のファイルは普通に開く
 *
 * @param const std::string &file
 * @retval std::istream * open先のstreamへのポインタ
 */
std::istream& InputFileOpen(const std::string& file);

/*!
 * @brief InputFileOpenで開いたファイルのclose
 * @param std::istream& in_fs
 * @retval void
 */
void InputFileClose(const std::istream& in_fs);

/*!
 * @brief 与えられた文字列を全て小文字にする
 * @param std::string& str
 * @retval void
 */
void ToLower(std::string& str);

/*!
 * @brief 与えられた文字列を全て大文字にする
 * @param std::string& str
 * @retval void
 */
void ToUpper(std::string& str);

/*!
 * @brief 与えられた文字列から与えられた文字列を消去する
 * @param std::string& str 操作対象の文字列
 * @param const std::string& rm 消去したい文字列
 * @retval void
 */
void RemoveStr(std::string& str, const std::string& rm);

/*!
 * @brief エラーメッセージを表示して終了する
 * @param const std::string& errmsg
 * @retval void
 */
void ExitErrMessage(const std::string& errmsg);

/*!
 * @brief 入力ストリームに文字列を書き戻す
 * @param std::istream& in_fs 入力ストリーム
 * @param std::string& str 書き戻す文字列
 * @retval std::istream& 文字列を書き戻した後の入力ストリーム
 */
std::istream& PutbackString(std::istream& in_fs, std::string& str);

template <class T, class S>
T lexical_cast(const S& source)
{
        T target;
        std::stringstream interpreter;
        if (!(interpreter << source && interpreter >> target)) {
                ExitErrMessage("Can't convert");
        }
        return target;
}
}

#endif // #ifndef _UTILITY_
